@@ -36,7 +36,11 @@
 #include "caml/sys.h"
 #include "caml/spacetime.h"
 
-#include "../config/s.h"
+#include "caml/s.h"
+
+#define SPACETIME_PROFINFO_WIDTH 26
+#define Spacetime_profinfo_hd(hd) \
+  (Gen_profinfo_hd(SPACETIME_PROFINFO_WIDTH, hd))
 
 #ifdef ARCH_SIXTYFOUR
 
@@ -48,7 +52,7 @@
 c_node* caml_spacetime_offline_c_node_of_stored_pointer_not_null
           (value node_stored)
 {
-  Assert(Is_c_node(node_stored));
+  CAMLassert(Is_c_node(node_stored));
   return (c_node*) Hp_val(node_stored);
 }
 
@@ -60,8 +64,8 @@ c_node_type caml_spacetime_offline_classify_c_node(c_node* node)
 CAMLprim value caml_spacetime_compare_node(
       value node1, value node2)
 {
-  Assert(!Is_in_value_area(node1));
-  Assert(!Is_in_value_area(node2));
+  CAMLassert(!Is_in_value_area(node1));
+  CAMLassert(!Is_in_value_area(node2));
 
   if (node1 == node2) {
     return Val_long(0);
@@ -85,19 +89,19 @@ CAMLprim value caml_spacetime_node_num_header_words(value unit)
 
 CAMLprim value caml_spacetime_is_ocaml_node(value node)
 {
-  Assert(Is_ocaml_node(node) || Is_c_node(node));
+  CAMLassert(Is_ocaml_node(node) || Is_c_node(node));
   return Val_bool(Is_ocaml_node(node));
 }
 
 CAMLprim value caml_spacetime_ocaml_function_identifier(value node)
 {
-  Assert(Is_ocaml_node(node));
+  CAMLassert(Is_ocaml_node(node));
   return caml_copy_int64((uint64_t) Decode_node_pc(Node_pc(node)));
 }
 
 CAMLprim value caml_spacetime_ocaml_tail_chain(value node)
 {
-  Assert(Is_ocaml_node(node));
+  CAMLassert(Is_ocaml_node(node));
   return Tail_link(node);
 }
 
@@ -107,7 +111,7 @@ CAMLprim value caml_spacetime_classify_direct_call_point
   uintnat field;
   value callee_node;
 
-  Assert(Is_ocaml_node(node));
+  CAMLassert(Is_ocaml_node(node));
 
   field = Long_val(offset);
 
@@ -127,14 +131,14 @@ CAMLprim value caml_spacetime_ocaml_allocation_point_annotation
 {
   uintnat profinfo_shifted;
   profinfo_shifted = (uintnat) Alloc_point_profinfo(node, Long_val(offset));
-  return Val_long(Profinfo_hd(profinfo_shifted));
+  return Val_long(Spacetime_profinfo_hd(profinfo_shifted));
 }
 
 CAMLprim value caml_spacetime_ocaml_allocation_point_count
       (value node, value offset)
 {
   value count = Alloc_point_count(node, Long_val(offset));
-  Assert(!Is_block(count));
+  CAMLassert(!Is_block(count));
   return count;
 }
 
@@ -144,26 +148,32 @@ CAMLprim value caml_spacetime_ocaml_direct_call_point_callee_node
   return Direct_callee_node(node, Long_val(offset));
 }
 
+CAMLprim value caml_spacetime_ocaml_direct_call_point_call_count
+(value node, value offset)
+{
+  return Direct_call_count(node, Long_val(offset));
+}
+
 CAMLprim value caml_spacetime_ocaml_indirect_call_point_callees
       (value node, value offset)
 {
   value callees = Indirect_pc_linked_list(node, Long_val(offset));
-  Assert(Is_block(callees));
-  Assert(Is_c_node(callees));
+  CAMLassert(Is_block(callees));
+  CAMLassert(Is_c_node(callees));
   return callees;
 }
 
 CAMLprim value caml_spacetime_c_node_is_call(value node)
 {
   c_node* c_node;
-  Assert(node != (value) NULL);
-  Assert(Is_c_node(node));
+  CAMLassert(node != (value) NULL);
+  CAMLassert(Is_c_node(node));
   c_node = caml_spacetime_offline_c_node_of_stored_pointer_not_null(node);
   switch (caml_spacetime_offline_classify_c_node(c_node)) {
     case CALL: return Val_true;
     case ALLOCATION: return Val_false;
   }
-  Assert(0);
+  CAMLassert(0);
   return Val_unit;  /* silence compiler warning */
 }
 
@@ -171,18 +181,18 @@ CAMLprim value caml_spacetime_c_node_next(value node)
 {
   c_node* c_node;
 
-  Assert(node != (value) NULL);
-  Assert(Is_c_node(node));
+  CAMLassert(node != (value) NULL);
+  CAMLassert(Is_c_node(node));
   c_node = caml_spacetime_offline_c_node_of_stored_pointer_not_null(node);
-  Assert(c_node->next == Val_unit || Is_c_node(c_node->next));
+  CAMLassert(c_node->next == Val_unit || Is_c_node(c_node->next));
   return c_node->next;
 }
 
 CAMLprim value caml_spacetime_c_node_call_site(value node)
 {
   c_node* c_node;
-  Assert(node != (value) NULL);
-  Assert(Is_c_node(node));
+  CAMLassert(node != (value) NULL);
+  CAMLassert(Is_c_node(node));
   c_node = caml_spacetime_offline_c_node_of_stored_pointer_not_null(node);
   return caml_copy_int64((uint64_t) Decode_c_node_pc(c_node->pc));
 }
@@ -190,38 +200,51 @@ CAMLprim value caml_spacetime_c_node_call_site(value node)
 CAMLprim value caml_spacetime_c_node_callee_node(value node)
 {
   c_node* c_node;
-  Assert(node != (value) NULL);
-  Assert(Is_c_node(node));
+  CAMLassert(node != (value) NULL);
+  CAMLassert(Is_c_node(node));
   c_node = caml_spacetime_offline_c_node_of_stored_pointer_not_null(node);
-  Assert(caml_spacetime_offline_classify_c_node(c_node) == CALL);
+  CAMLassert(caml_spacetime_offline_classify_c_node(c_node) == CALL);
   /* This might be an uninitialised tail call point: for example if an OCaml
      callee was indirectly called but the callee wasn't instrumented (e.g. a
      leaf function that doesn't allocate). */
-  if (Is_tail_caller_node_encoded(c_node->data.callee_node)) {
+  if (Is_tail_caller_node_encoded(c_node->data.call.callee_node)) {
     return Val_unit;
   }
-  return c_node->data.callee_node;
+  return c_node->data.call.callee_node;
+}
+
+CAMLprim value caml_spacetime_c_node_call_count(value node)
+{
+  c_node* c_node;
+  CAMLassert(node != (value) NULL);
+  CAMLassert(Is_c_node(node));
+  c_node = caml_spacetime_offline_c_node_of_stored_pointer_not_null(node);
+  CAMLassert(caml_spacetime_offline_classify_c_node(c_node) == CALL);
+  if (Is_tail_caller_node_encoded(c_node->data.call.callee_node)) {
+    return Val_long(0);
+  }
+  return c_node->data.call.call_count;
 }
 
 CAMLprim value caml_spacetime_c_node_profinfo(value node)
 {
   c_node* c_node;
-  Assert(node != (value) NULL);
-  Assert(Is_c_node(node));
+  CAMLassert(node != (value) NULL);
+  CAMLassert(Is_c_node(node));
   c_node = caml_spacetime_offline_c_node_of_stored_pointer_not_null(node);
-  Assert(caml_spacetime_offline_classify_c_node(c_node) == ALLOCATION);
-  Assert(!Is_block(c_node->data.allocation.profinfo));
-  return Val_long(Profinfo_hd(c_node->data.allocation.profinfo));
+  CAMLassert(caml_spacetime_offline_classify_c_node(c_node) == ALLOCATION);
+  CAMLassert(!Is_block(c_node->data.allocation.profinfo));
+  return Val_long(Spacetime_profinfo_hd(c_node->data.allocation.profinfo));
 }
 
 CAMLprim value caml_spacetime_c_node_allocation_count(value node)
 {
   c_node* c_node;
-  Assert(node != (value) NULL);
-  Assert(Is_c_node(node));
+  CAMLassert(node != (value) NULL);
+  CAMLassert(Is_c_node(node));
   c_node = caml_spacetime_offline_c_node_of_stored_pointer_not_null(node);
-  Assert(caml_spacetime_offline_classify_c_node(c_node) == ALLOCATION);
-  Assert(!Is_block(c_node->data.allocation.count));
+  CAMLassert(caml_spacetime_offline_classify_c_node(c_node) == ALLOCATION);
+  CAMLassert(!Is_block(c_node->data.allocation.count));
   return c_node->data.allocation.count;
 }
 
