@@ -404,7 +404,9 @@ let read_one_param ppf position name v =
   | "can-discard" ->
     can_discard := v ::!can_discard
 
-  | "timings" -> set "timings" [ print_timings ] v
+  | "timings" | "profile" ->
+     let if_on = if name = "timings" then [ `Time ] else Profile.all_columns in
+     profile_columns := if check_bool ppf name v then if_on else []
 
   | "plugin" -> !load_plugin v
 
