@@ -52,15 +52,7 @@ type ('a, 'b) foo = ..
 type ('a, 'b) bar = ('a, 'a) foo = .. (* Error: constraints do not match *)
 ;;
 
-(* Private abstract types cannot be open *)
-
-type foo = ..
-;;
-
-type bar = private foo = .. (* ERROR: Private abstract types cannot be open *)
-;;
-
-(* Check that signatures can hide open-ness *)
+(* Check that signatures can hide exstensibility *)
 
 module M = struct type foo = .. end
 ;;
@@ -74,7 +66,7 @@ module M_S = (M : S)
 type M_S.foo += Foo (* ERROR: Cannot extend a type that isn't "open" *)
 ;;
 
-(* Check that signatures cannot add open-ness *)
+(* Check that signatures cannot add extensibility *)
 
 module M = struct type foo end
 ;;
@@ -85,6 +77,32 @@ module type S = sig type foo = .. end
 module M_S = (M : S) (* ERROR: Signatures are not compatible *)
 ;;
 
+(* Check that signatures can make exstensibility private *)
+
+module M = struct type foo = .. end
+;;
+
+module type S = sig type foo = private .. end
+;;
+
+module M_S = (M : S)
+;;
+
+type M_S.foo += Foo (* ERROR: Cannot extend a private extensible type *)
+;;
+
+(* Check that signatures cannot make private extensibility public *)
+
+module M = struct type foo = private .. end
+;;
+
+module type S = sig type foo = .. end
+;;
+
+module M_S = (M : S) (* ERROR: Signatures are not compatible *)
+;;
+
+
 (* Check that signatures maintain variances *)
 
 module M = struct type +'a foo = .. type 'a bar = 'a foo = .. end
